#!/usr/bin/python
# Make the thumbnail file "index.html" for images in all immediate subdirectories
import sys
import os
import string
import urllib
import Image
import ImageDraw
    

def myexec(cmd,args):
    """Execute the command"""
    ret = os.spawnvp(os.P_WAIT,cmd,[cmd]+args)
    if (ret!=0):
        print("error: executing "+str([cmd]+args)+" returned "+str(ret)+"\n")
        sys.exit(1)

def thumbnail(fn):
    """Makes an 80x80 thumbnail out of the fn, and returns it as an im"""
    print(fn)
    im = Image.open(fn)
    if (im.mode!="RGB"):
        im = im.convert("RGB")
    width,height = im.size
    # a roughly-aspect-preserving scale so the minimum dimension is 80pixels
    cropsize = 80
    scalew,scaleh = (float(cropsize)/width, float(cropsize)/height)
    scale = max(scalew,scaleh)
    sw,sh = (min(int(width*scale+0.5),cropsize*3), min(int(height*scale+0.5),cropsize*3))
    im.thumbnail((sw,sh), Image.ANTIALIAS)
    # now crop it to the central 80x80
    cx,cy = (int((sw-cropsize)/2), int((sh-cropsize)/2) )
    im2 = im.crop( (cx,cy,cx+cropsize,cy+cropsize) )
    del im
    return im2


def titleblock(caption):
    """Draws an 80x80 block with the specified text centered in it"""
    im = Image.new("RGB",(80,80))
    draw = ImageDraw.Draw(im)
    # caption is a list of nonblanks separated by blanks.
    words = caption.split()
    # for each word calculate its size
    szwords = []
    height = 0
    for s in words:
        (w,h) = draw.textsize(s)
        szwords.append( (w,h,s) )
        if (h>height):
            height=h
    (sepw,seph) = draw.textsize(" ")
    # build up rows
    rows = []
    row = []
    pos = 0
    for (w,h,s) in szwords:
        if (pos==0):
            row += [(w,s)]
            pos += w
        elif (pos+sepw+w<80):
            row += [(sepw," "),(w,s)]
            pos += sepw+w
        else:
            rows += [(pos,row)]
            row = [(w,s)]
            pos = w
    if (pos>0):
        rows += [(pos,row)]
    # figure out heights
    numrows = len(rows)
    heightall = numrows*height
    y = (80-heightall)/2
    # now draw each word
    for (rw,row) in rows:
        x = (80-rw)/2
        for (w,s) in row:
            draw.text( (x,y),s )
            x += w
        y += height
    del draw
    return im
    

def map(caption,width,height,ifiles,jpegfn,anchor):
    """Makes an imagemap out of fns, saves it to file mapfn, and returns a <map>...</map>"""
    im = Image.new("RGB",(width*80,height*80))
    s = "<map name=\""+anchor+"\">\n"
    x,y = 0,0
    for fn in ifiles:
        try:
            if (fn=="#index"):
                tim = titleblock(caption)
                fn = caption+"/"
            else:
                tim = thumbnail(fn)
        except IOError:
            tim = Image.new("RGB",(80,80))
        im.paste(tim,(x*80,y*80))
        del tim
        s += "<area onclick=\"return c(this,event)\" href=\""+urllib.quote(fn)+"\" shape=rect coords=\""+str(x*80)+","+str(y*80)+","+str(x*80+79)+","+str(y*80+79)+"\">\n"
        x += 1
        if (x>=12):
            x=0
            y += 1
    im.save(jpegfn,"JPEG",quality=40)
    s += "</map>\n"
    return s
        


def dodir(dir, files, pos):
    """Ensure that the directory's thumbnails are there, and return an integer for the following dir's pos
    Dir is the directory to examine. HTML we generate will point to that directory. files is a list of
    the files in that directory. Pos is where we start. We return a tuple (shtml,int) for the html
    to display this, and the pos for the next one. As a side effect we create some files in the directory."""
    # was it already there?
    prefix = dir+"/.thumb."+str(pos)
    (dir2,caption) = os.path.split(dir)
    if (os.path.exists(prefix+".shtml") and os.path.exists(prefix+".next")):
        f = file(prefix+".next","r")
        p2 = f.read()
        f.close()
        f = file(prefix+".shtml","r")
        shtml = f.read()
        f.close()
        return (shtml, int(p2))
    # if not, we have to generate it
    print dir+"."+str(pos)
    # first figure out all the filenames
    ifiles = ["#index"]
    for fn in files:
        root,ext = os.path.splitext(fn)
        ext = string.lower(ext)
        if (ext!=".jpg" and ext!=".jpeg" and ext!=".gif" and ext!=".tif" and ext!=".tiff" and ext!=".bmp"):
            continue
        if (fn[0:1]=="."):
            continue
        ifiles.append(dir+"/"+fn)
    # figure out how we'll build our three chunks
    numthumbs = len(ifiles)
    if (pos==0 or numthumbs+pos<=12):
        numa = 0
    else:
        numa = 12-pos
    numthumbs-=numa
    numc = numthumbs % 12
    numb = numthumbs - numc
    numbrows = numb/12
    nextpos = numc
    if (numa==0 and numb==0):
        nextpos = (pos+numc)%12
    ifilesa = ifiles[0:numa]
    ifilesb = ifiles[numa:numa+numb]
    ifilesc = ifiles[numa+numb:]
    # and build them
    shtml = ""
    if (numa>0):
        anchor,tfn = (prefix+".a", prefix+".a.jpg")
        shtml += "<img ismap usemap=\"#"+anchor+"\" src=\""+tfn+"\" border=0 width="+str(80*numa)+" height=80/><br>\n"
        shtml += map(caption,numa,1,ifilesa,tfn,anchor)
    if (numb>0):
        anchor,tfn = (prefix+".b", prefix+".b.jpg")
        shtml += "<img ismap usemap=\"#"+anchor+"\" src=\""+tfn+"\" border=0 width="+str(80*12)+" height="+str(80*numbrows)+"/><br>\n"
        shtml += map(caption,12,numbrows,ifilesb,tfn,anchor)
    if (numc>0):
        anchor,tfn = (prefix+".c", prefix+".c.jpg")
        shtml += "<img ismap usemap=\"#"+anchor+"\" src=\""+tfn+"\" border=0 width="+str(80*numc)+" height=80/>"
        shtml += map(caption,numc,1,ifilesc,tfn,anchor).replace("\n","")
    # save the output for future
    f = file(prefix+".shtml","w")
    f.write(shtml)
    f.close()
    f = file(prefix+".next","w")
    f.write(str(nextpos))
    f.close()
    return (shtml,nextpos)
    


def main():
    pos = 0
    html = "<html>\n<head>\n<title>Pictures</title>\n<script type=\"text/javascript\">\nfunction c(o,e){if (e.shiftKey||e.ctrlKey)return;location.href='slideshow.html?dir='+o.href; return false;}\n</script>\n<body style=\"background-color:black;\">\n<div style=\"width: 960px; margin-left:auto; margin-right:auto;\">\n"
    for dir,dirs,files in os.walk("."):
        dirs.sort()
        files.sort()
        dir = os.path.normpath(dir); #remove leading "./"
        (head,tail) = os.path.split(dir)
        if (dir=="" or tail[:1]=="."):
            continue
        (shtml,pos2) = dodir(dir,files,pos)
        pos = pos2
        html += shtml
    html += "\n</div></body></html>\n"
    f = file("thumbs.html","w")
    f.write(html)
    f.close()
    print("thumbs.html")

slideshow_src = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title>Slideshow</title>
<meta http-equiv="Content-Type" content="text/html;charset=utf-8"/>
<style type="text/css"> html, body, #pic {width:100%;height:100%;}</style>
</head>
<body style="color:white; background-color:black; margin:0; overflow:hidden;">

<div id="pic"></div>
<div id="pic2" style="position:fixed; right:20px; top:20px; width:100px; height:100px; overflow:hidden;"></div>


<div style="color:rgb(100,100,100); position:fixed; top:0px; right:6px;
  font-family:'Courier New'; font-weight:bold; font-size: smaller;"
  onmouseover="this.style.backgroundColor='black';"
  onmouseout="this.style.backgroundColor='transparent';"><span
  style="cursor:pointer;" onmousedown="next(-1);">&lt;</span>&nbsp;<span
  id="middle" style="cursor:pointer;"
  onmousedown="middle();">&#9654;</span>&nbsp;<span 
  style="cursor:pointer;" onmousedown="next(1);">&gt;</span></div>



<script type="text/javascript">
var xmlhttp;
try {xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");}
catch(e) {xmlhttp = new XMLHttpRequest();}
var path = urlsplit(location.href);
path = decodeURIComponent(path.query.split("?dir=")[1].replace(/\+/g," "));
path = urlsplit(path);
file = decodeURI(path.file);
path = path.site+path.path;
var files=[], titles=[];
var fileindex=-1;
var iw=-1, ih=-1;
var auto=true;
var tt=null;
window.onresize = resize;
document.onmousedown=mousedown;

function middle()
{ if (tt) clearTimeout(tt); tt=null;
  if (auto) {auto=false; fixcontrols(); return;}
  else {auto=true; timer();}
}

function next(i)
{ if (tt) clearTimeout(tt); tt=null; auto=false;
  if (fileindex==-1 || files.length==0) {show(); return;}
  fileindex+=i; if (fileindex<0) fileindex=files.length-1;
  if (fileindex>=files.length) fileindex=0;
  show();
}


function timer()
{ if (!auto) return;
  if (fileindex==-1 || files.length==0) return;
  fileindex++; if (fileindex>=files.length) fileindex=0;
  show();
}

function fixcontrols()
{ var m = document.getElementById("middle");
  if (auto) m.innerHTML='&#9633;';
  else m.innerHTML='&#9658;';
}

function show() // called when it's time to load image number 'fileindex'
{ fixcontrols();
  if (fileindex==-1) {document.title="Slideshow"; return;}
  document.getElementById("pic2").innerHTML = '<img id="img2" src="'+files[fileindex]+'" style="width:100%;height:100%;" onload="loaded();"/>';
  document.getElementById("pic2").style.display='';
}

function loaded() // called when the image is loaded
{ var i = document.getElementById("img2");
  i.style.width='auto'; i.style.height='auto';
  iw=i.width; ih=i.height;
  document.getElementById("pic2").innerHTML = '';
  document.getElementById("pic2").style.display='none';
  var p = document.getElementById("pic");
  p.style.width='100%'; p.style.height='100%';
  p.innerHTML = '<img id="img" src="'+files[fileindex]+'" style="width:0px; height:0px;"/>';
  resize();
  document.title = "["+(fileindex+1)+"/"+files.length+"] "+titles[fileindex];
  if (tt) clearTimeout(tt); tt=null;
  if (auto) tt=setTimeout(timer,6000);
}

function resize() // called when the window resizes
{ var p = document.getElementById("pic");
  var i = document.getElementById("img");
  if (p.style.width=='auto') return;
  var ww=p.offsetWidth;
  var wh=p.offsetHeight;
  var wr=ww/wh, ir=iw/ih;
  var nih,niw;
  if (ir>wr) {nih=Math.floor(ww/ir); niw=ww;}
  else {niw=Math.floor(wh*ir); nih=wh;}
  i.style.width=niw+"px";
  i.style.height=nih+"px";
  i.style.marginLeft=Math.floor((ww-niw)/2)+"px";
  i.style.marginTop=Math.floor((wh-nih)/2)+"px";
}

var dragx=0,dragy=0, hasdragged=false, firstdown=true;
function mousedown(event)
{ if (!event) event=window.event;
  var target = event.target || event.srcElement;
  if (target.id!="img") return;
  var p = document.getElementById("pic");
  var i = document.getElementById("img");
  if (p.style.width!='auto')
  { if (auto) middle(); // to turn of the auto-slideshow
    var ox=parseInt(i.style.marginLeft), oy=parseInt(i.style.marginTop);
    var ow=parseInt(i.style.width), oh=parseInt(i.style.height);
    var fx=Math.floor((event.clientX-ox)/ow*iw), fy=Math.floor((event.clientY-oy)/oh*ih);
    p.style.width='auto'; p.style.height='auto';
    i.style.width='auto'; i.style.height='auto';
    i.style.marginLeft=(event.clientX-fx)+'px'; i.style.marginTop=(event.clientY-fy)+'px';
    i.style.cursor='pointer'; firstdown=true;
  }
  dragx=event.clientX-parseInt(i.style.marginLeft);
  dragy=event.clientY-parseInt(i.style.marginTop);
  isdragging=true; hasdragged=false;
  document.onmousemove=mousemove;
  document.onmouseup=mouseup;
  return false;
}

function mousemove(event)
{ if (!event) event=window.event;
  var x=event.clientX, y=event.clientY; 
  hasdragged=true;
  document.getElementById("img").style.marginLeft=(x-dragx)+"px";
  document.getElementById("img").style.marginTop=(y-dragy)+"px";
  return false;
}

function mouseup(event)
{ var wasfirstdown=firstdown;
  isdragging=false; firstdown=false;
  document.onmousemove=null; document.onmouseup=null;
  if (hasdragged || wasfirstdown) return;
  var p = document.getElementById("pic");
  var i = document.getElementById("img");
  p.style.width='100%'; p.style.height='100%'; resize(); return;
  i.style.cursor='';
}


xmlhttp.open('get',path,true);
xmlhttp.onreadystatechange = function()
{ if (xmlhttp.readyState!=4) return;
  if (xmlhttp.status!=200) {alert(striptags(xmlhttp.responseText)); return;}
  var s = xmlhttp.responseText;
  files = s.match(/<a\s*href=(("[^">]*?\.((jpe?g)|(gif))")|('[^'>]*?\.((jpe?g)|(gif))')|([^\s]*?\.((jpe?g)|(gif))))[^>]*>/gi);
  fileindex=-1;
  if (files) for (var i=0; i<files.length; i++)
  { var fn = files[i].replace(/^<a\s*href=(("[^">]*?\.((jpe?g)|(gif))")|('[^'>]*?\.((jpe?g)|(gif))')|([^\s]*?\.((jpe?g)|(gif))))[^>]*>/gi,"$1");
    fn = fn.replace(/^"([^"]*)"$/,"$1").replace(/^'([^']*)'$/,"$1");
    files[i] = path+fn;
    titles[i] = decodeURI(fn);
    if (file==titles[i]) fileindex=i;
  }
  if (file!="" && fileindex==-1) alert("Not found "+file+" in "+dump(titles));
  if (fileindex==-1) fileindex=0;
  auto=false; show();
}
xmlhttp.send(null);


function urlsplit(url)
{ var m = /((http|file):\/\/[^\/]*)?([^\?]*\/)?([^\?\/]*)(\?.*)?/i.exec(url);
  if (!m) return null; else return {'site':m[1]?m[1]:'', 'path':m[3]?m[3]:'', 'file':m[4]?m[4]:'', 'query':m[5]?m[5]:''};
}

function dump(o)
{ var s="";
  for (var i in o)
  { var v = o[i];
    if (typeof v=='function') s+=i+"=fn ";
    else if (i.substring(0,2)=="on" && o[i]==null) s+="";
    else
    { v=v+""; s+=i+"=";
      if (v.length>100) s+=v.substring(0,100)+"... ";
      else s+=v+" ";
    }
  }
  return s;
}

function striptags(html)
{ return html.replace(/<[^>]+>/ig,' ');
}

</script>

</body>
</html>
"""

try:
    if not os.path.exists("slideshow.html"):
        f = open("slideshow.html","w")
        f.write(slideshow_src)
        f.close()
        print("slideshow.html")
    main()
except KeyboardInterrupt:
    print("");
